///////////////////////////////////////////////////////////////////////////////////////////////////
// Company: <Name>
//
// File: Ethernet10G_Sub_Tb.v
// File history:
//      <Revision number>: <Date>: <Comments>
//      <Revision number>: <Date>: <Comments>
//      <Revision number>: <Date>: <Comments>
//
// Description: 
//
// <Description here>
//
// Targeted device: <Family::PolarFire> <Die::MPF300TS_ES> <Package::FCG1152>
// Author: <Name>
//
/////////////////////////////////////////////////////////////////////////////////////////////////// 

`timescale 1ns/1ps

module Ethernet10G_Sub_Tb;

parameter APBCLK_PERIOD = 10; // 100MHZ
parameter SYSCLK_PERIOD = 6.4;// 156.25MHZ

reg SYSCLK;
wire APBCLK;
reg NSYSRESET;

wire LANE_TXD_P;
wire LANE_TXD_N;

/* APB 10G */
reg         PWRITE;
reg  [15:0] PADDR;
reg         PSEL;
reg         PENABLE;
reg  [31:0] PWDATA;
wire [31:0] PRDATA;
wire        PREADY;
wire        PSLVERR;

reg [31:0]  eth_rdata;

/* APB FIFO Inteface */
reg         PWRITE_F;
reg  [31:0] PADDR_F;
reg         PSEL_F;
reg         PENABLE_F;
reg  [31:0] PWDATA_F;
wire [31:0] PRDATA_F;
wire        PREADY_F;
//wire        PSLVERR_F;
reg [31:0]  fifo_rdata;

wire        O_SYS_MAC_RX_RDY;

initial
begin
    SYSCLK = 1'b0;
    NSYSRESET = 1'b0;
end

//////////////////////////////////////////////////////////////////////
// Reset Pulse
//////////////////////////////////////////////////////////////////////
initial
begin
    #(SYSCLK_PERIOD * 10 )
        NSYSRESET = 1'b1;
end


//////////////////////////////////////////////////////////////////////
// Clock Driver
//////////////////////////////////////////////////////////////////////
always @(SYSCLK)
    #(SYSCLK_PERIOD / 2.0) SYSCLK <= !SYSCLK;


//////////////////////////////////////////////////////////////////////
// Instantiate Unit Under Test:  Ethernet10G_Subsystem
//////////////////////////////////////////////////////////////////////
Ethernet10G_Subsystem Ethernet10G_Subsystem_0 (
    // Inputs
    .LANE0_RXD_P(LANE_TXD_P),
    .LANE0_RXD_N(LANE_TXD_N),
    .REF_CLK_PAD_P(SYSCLK),
    .REF_CLK_PAD_N(~SYSCLK),
    .SYS_TX_RESET(NSYSRESET),
    .SYS_RX_RESET(NSYSRESET),
    .PSEL_IN(PSEL_F),
    .PENABLE_IN(PENABLE_F),
    .PWRITE_IN(PWRITE_F),
    .PENABLE(PENABLE),
    .PWRITE(PWRITE),
    .PSEL(PSEL),
    .PADDR_IN(PADDR_F),
    .PWDATA_IN(PWDATA_F),
    .PADDR(PADDR),
    .PWDATA(PWDATA),

    // Outputs
    .LANE0_TXD_P(LANE_TXD_P),
    .LANE0_TXD_N(LANE_TXD_N),
    .clk_100mhz(APBCLK),
    .PRESETN( ),
    .Retime_1_Q( ),
    .Retime_0_Q( ),
    .PF_CCC_1_LOCK( ),
    .O_SYS_MAC_RX_RDY(O_SYS_MAC_RX_RDY),
    .LANE0_RX_READY( ),
    .O_SYS_MAC_TX_RDY( ),
    .O_CORE_TX_SRESET( ),
    .O_CORE_RX_SRESET( ),
    .TX_PLL_LOCK( ),
    .PREADY_OUT(PREADY_F),
    .O_SYS_MAC_TX_FAULT( ),
    .O_SYS_MAC_RX_ERR_FIFO_OVERFLOW( ),
    .O_SYS_MAC_RX_ERR( ),
    .O_SYS_MAC_TX_ERR_FIFO_OVERFLOW( ),
    .O_SYS_MAC_TX_ERR_FIFO_UNDERRUN( ),
    .PREADY(PREADY),
    .PSLVERR(PSLVERR),
    .PRDATA_OUT(PRDATA_F),
    .PRDATA(PRDATA)

    // Inouts

);

// 10GMAC APB REG Configuaration
   initial 
    begin
	$display("\n");
	$display("\n");
	$display("Core10GMAC REG configuration \n");
	wait (O_SYS_MAC_RX_RDY == 1);

    cpu_read (((16'hB0)<<2),eth_rdata);
    cpu_read (((16'hB3)<<2),eth_rdata);
    cpu_read (((16'hA3)<<2),eth_rdata);
    cpu_read (((16'hA4)<<2),eth_rdata);

    cpu_write ((16'h28C),32'h1C000);
    cpu_write ((16'h290),32'h10C);
    cpu_write ((16'h2C0),32'h8);
    cpu_write ((16'h2CC),32'h1C000);

    cpu_read ((16'h28C),eth_rdata);
    cpu_read ((16'h290),eth_rdata);
    cpu_read ((16'h2C0),eth_rdata);
    cpu_read ((16'h2CC),eth_rdata);

    // delay for 1 us
    #10000;

    // write to Fifo
    cpu_write_fifo ((32'h4),32'hFFFFFFFF);
    cpu_write_fifo ((32'h0),32'h0202FFFF);
    cpu_write_fifo ((32'h8),32'h00000200);

    cpu_write_fifo ((32'h4),32'h02020202);
    cpu_write_fifo ((32'h0),32'h01000608);
    cpu_write_fifo ((32'h8),32'h00000000);
    cpu_write_fifo ((32'h4),32'h04060008);
    cpu_write_fifo ((32'h0),32'h02020100);
    cpu_write_fifo ((32'h8),32'h00000000);
    cpu_write_fifo ((32'h4),32'h02020202);
    cpu_write_fifo ((32'h0),32'hAF82A8C0);
    cpu_write_fifo ((32'h8),32'h00000000);
    cpu_write_fifo ((32'h4),32'hFFFFFFFF);
    cpu_write_fifo ((32'h0),32'hA8C0FFFF);
    cpu_write_fifo ((32'h8),32'h00000000);
    cpu_write_fifo ((32'h4),32'h0000AF82);
    cpu_write_fifo ((32'h0),32'h00000000);
    cpu_write_fifo ((32'h8),32'h00000000);
    cpu_write_fifo ((32'h4),32'h00000000);
    cpu_write_fifo ((32'h0),32'h00000000);
    cpu_write_fifo ((32'h8),32'h00000000);
    cpu_write_fifo ((32'h4),32'h00000000);
    cpu_write_fifo ((32'h0),32'h00000000);
    cpu_write_fifo ((32'h8),32'h0000010F);

    end



// This task does a cpu write 
task cpu_write;
input [15:0] address;
input [31:0] data;
begin
  $display("CPU Write %04x = %04x",address,data);
  @(negedge APBCLK);  
  PWRITE  = 1'b0;
  @(negedge APBCLK);  
  PENABLE  = 1'b0;
  PSEL  = 1'b1;
  PADDR = address;
  PWDATA  = data;  
  PWRITE  = 1'b1;
  @(negedge APBCLK);  
  PENABLE  = 1'b1;
  @(negedge APBCLK);  
  while (PREADY==1'b0) @(negedge APBCLK);
  PWRITE  = 1'b0;
  PENABLE  = 1'b0;
  PSEL  = 1'b0;
end
endtask

// This task does a cpu read 
task cpu_read;
input [15:0] address;
output [31:0] data;
reg [31:0] data;
begin
  @(negedge APBCLK);  
  PWRITE  = 1'b1;
  @(negedge APBCLK);  
  PENABLE  = 1'b0;
  PSEL  = 1'b1;
  PADDR = address;
  PWRITE  = 1'b0;
  @(negedge APBCLK);  
  PENABLE  = 1'b1;
  @(negedge APBCLK);
  while (PREADY==1'b0) @(negedge APBCLK);
  data = PRDATA;
  PWRITE  = 1'b1;
  PENABLE  = 1'b0;
  PSEL  = 1'b0;
  $display("CPU Read %04x = %04x",address,data);
end
endtask

// This task does a cpu write 
task cpu_write_fifo;
input [31:0] address;
input [31:0] data;
begin
  $display("CPU Write %04x = %04x",address,data);
  @(negedge APBCLK);  
  PWRITE_F  = 1'b0;
  @(negedge APBCLK);  
  PENABLE_F  = 1'b0;
  PSEL_F  = 1'b1;
  PADDR_F = address;
  PWDATA_F  = data;  
  PWRITE_F  = 1'b1;
  @(negedge APBCLK);  
  PENABLE_F  = 1'b1;
  @(negedge APBCLK);  
  while (PREADY_F==1'b0) @(negedge APBCLK);
  PWRITE_F  = 1'b0;
  PENABLE_F  = 1'b0;
  PSEL_F  = 1'b0;
end
endtask

// This task does a cpu read 
task cpu_read_fifo;
input [31:0] address;
output [31:0] data;
reg [31:0] data;
begin
  @(negedge APBCLK);  
  PWRITE_F  = 1'b1;
  @(negedge APBCLK);  
  PENABLE_F  = 1'b0;
  PSEL_F  = 1'b1;
  PADDR_F = address;
  PWRITE_F  = 1'b0;
  @(negedge APBCLK);  
  PENABLE_F  = 1'b1;
  @(negedge APBCLK);
  while (PREADY_F==1'b0) @(negedge APBCLK);
  data = PRDATA_F;
  PWRITE_F  = 1'b1;
  PENABLE_F  = 1'b0;
  PSEL_F  = 1'b0;
  $display("CPU Read %04x = %04x",address,data);
end
endtask


endmodule

